#!/usr/bin/perl -w
use strict;
use Bio::SearchIO; 
use Data::Dumper;
use Getopt::Long;


# Run blastall 
# blastall -p blastp -M BLOSUM62 -m7 -d GH13full.fasta -i TargetGenes/targetseqs_gi_121603144.fsa > test.blastout

#------------------------------------------------------------------------
# Get options
#------------------------------------------------------------------------
my ($queryfile, $queryfile2, @queryfiles, $databasefile, $blastout);

&GetOptions ( "db:s" =>  \$databasefile);
@queryfiles = @ARGV;

unless (-e $queryfiles[0] and defined $databasefile) {
print "#===================== ERROR: ============================\n";	
	print "# Database or query file no defined or does not exist\n";
	print "# Does the query file exist?\n";
print "#===================== Usage: ============================\n";	
	print "# All files that match pattern (target*) are used as queries\n";	
	print "# perl targetOrganismBLASTwrapper -db GH13full target*\n";
	print "# Specific file used as query\n";	
	print "# perl targetOrganismBLASTwrapper -db GH13full TargetGenes/targetseqs_gi_121603144.fsa\n";		
print "#=========================================================\n";	
	exit;
}

#------------------------------------------------------------------------
# Print information to user
#-----------------------------------------------------------------------

print "#===================== Variable values: ============================\n";	
print "# Query files: ", scalar(@queryfiles)," files will be used\n";
print "# Database: $databasefile\n";
print "#=========================================================\n";	
#========================================================================
# MAIN
#========================================================================

system("formatdb -p T -i $databasefile");
my $count = 0;

for my $queryfile (@queryfiles){ 
	$count++;
	print "#=========================================================\n";
	print "# Running analysis of file $count of ", scalar(@queryfiles) ,"\n";
	$queryfile2 = (split("\/", $queryfile))[1] if ($queryfile =~ m/\//);
	$blastout = (split("\\.", $queryfile2))[0] . "_db" . (split("\\.", $databasefile))[0] . ".blastout";
	#print Dumper($queryfile, $queryfile2, $databasefile, $blastout);
	
	system ("blastall -p blastp -m7 -d $databasefile -i $queryfile -o $blastout ");

	my $in = new Bio::SearchIO(-format => 'blastxml', -file   => $blastout);
	my ($maxlength, $flag, $hitflag);
	$hitflag = 0;
	while( my $result = $in->next_result ) {
		$flag = 0;
		while( my $hit = $result->next_hit ) {
			if ($hit->length > $result->query_length) { $maxlength = $hit->length } 
			else { $maxlength = $result->query_length }
		
			while( my $hsp = $hit->next_hsp ) {
				if( $hsp->length('total') > $maxlength/2 and $hsp->percent_identity >= 50 and $flag == 0) {
					print "MATCH: Query=",   $result->query_name, "\t", $result->database_name, "\t",$hsp->length('total'), "\t", $hsp->percent_identity, "\n";
					#print "Query=",   $result->query_name, "\t", $result->database_name, "\t",$hsp->length('total'), "\t", $hsp->percent_identity, "\n";
					$flag =1;
					$hitflag = 1;
				}
			}  
		}
	}

	if ($hitflag == 0){ 
		print "# No acceptable hits\n" ; 
		$hitflag = 1;
	}

	print "#=========================================================\n";
}
